use nats::Connection;
use std::{
    io::{self, BufRead},
    process::exit,
};

use crate::chat_message::ChatMessage;

mod chat_message;
mod nats_cli;

fn main() {
    let result = run_chat_program();
    if let Err(e) = result {
        match e.kind() {
            io::ErrorKind::ConnectionRefused => {
                eprintln!("Connection to server is refused. Please check if it is really running.");
                exit(1);
            }
            io::ErrorKind::InvalidData => {
                if e.to_string().contains("Authorization Violation") {
                    eprintln!("Invalid login credentials.");
                    exit(1);
                } else {
                    panic!("NATS response was invalid: {}", e);
                }
            }
            _ => panic!("{:?}", e),
        }
    }
}

fn run_chat_program() -> io::Result<()> {
    let username = ask_user_name();
    // TODO: Ask password from user
    let password = username.clone();
    let nc = nats_cli::connect_to_nats(&username, &password)?;

    println!(
        "Hello {}, please write your message. Use q to quit:",
        username
    );

    let subscription = nats_cli::subscribe_to_chat_messages(username.clone(), &nc)?;

    user_message_writing_loop(&username, &nc)?;

    subscription.unsubscribe()?;
    Ok(())
}

fn ask_user_name() -> String {
    println!("Please enter your username: ");
    let mut input_text = String::new();
    io::stdin()
        .read_line(&mut input_text)
        .expect("Failed reading from stdin");
    input_text = input_text.trim().to_string();
    if input_text.is_empty() {
        println!("Your input was empty. Please try again.");
        return ask_user_name();
    }
    input_text
}

fn user_message_writing_loop(username: &str, nc: &Connection) -> io::Result<()> {
    let stdin = io::stdin();
    for line in stdin.lock().lines() {
        let line = line.unwrap();
        let line = line.trim();
        if line.is_empty() {
            continue;
        }

        if line == "q" || line == "Q" {
            break;
        }

        nats_cli::publish_message(
            &ChatMessage {
                author: username.to_string(),
                message: line.to_string(),
            },
            &nc,
        )?;
    }
    Ok(())
}
